/*
 * Szudoku program, amely a GDF részére diplomamunkául szolgál.
 * A programot írta: Pataki István, 2016-ban.
 */
package SudokuPIPkg.GUI;

import SudokuPIPkg.DataForms.DColors;
import SudokuPIPkg.DataForms.DCoord;
import SudokuPIPkg.Basis.IConst;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import java.awt.Font;

/*
 Ez az osztály kialakítja a cella képét, és kezeli annak tartalmát.
 A lényeg az volt, hogy úgy alakítsam ki a rácsot, azon belül a cellákat,hogy
 annnak minden elemét kezelni tudjam.
 */
public class Cell extends JPanel implements IConst {

  /**
   * Lehetséges értékeke számának tárolója.
   */
  private byte candidatesCounter = 9;

  /**
   * A jelöltértékek megjelenítésére szolgáló tömb.
   */
  private JLabel[] candidate = new JLabel[10]; // 1..9

  /**
   * A cella megoldás-értéke.
   */
  private JLabel result = new JLabel("");

  /**
   * rácson belüli pozíció.
   */
  private DCoord coord;

  /**
   * Paraméter nélküli konstruktor, mely alapértelmezett értékekkel meghívja az
   * osztály fő konstruktorát.
   */
  public Cell() {
    this(new DCoord((byte) 0, (byte) 0), new DColors());
  }

  /**
   * Paraméteres konstruktor.
   *
   * @param coord
   * @param colors
   */
  public Cell(DCoord coord, DColors colors) {
    init(coord, colors);
  }

  /**
   * Cella kialakítása.
   *
   * @param coord
   * @param colors
   */
  private void init(DCoord coord, DColors colors) {
    // UIManager.getLookAndFeelDefaults().put("defaultFont", new Font("Arial", Font.BOLD, 14));
    byte row, col;
    setLayout(null);
    for (byte cand = 1; cand < 10; cand++) {
      row = (byte) (((cand - 1) / 3) * 20 + 2);
      col = (byte) (((cand - 1) % 3) * 20 + 2);
      candidate[cand] = new JLabel("" + cand);
      candidate[cand].setBounds(col, row, 20, 20);
      candidate[cand].setHorizontalAlignment(SwingConstants.CENTER);
      candidate[cand].setFont(new Font("Times New Roman", Font.PLAIN, 18));
      add(candidate[cand]);
    }
    result.setHorizontalAlignment(SwingConstants.CENTER);
    result.setFont(new Font("Times New Roman", Font.PLAIN, 40));
    result.setBounds(10, 10, 40, 40);
    result.setVisible(false);
    add(result);
    setCoord(coord);
    repaintColors(colors);
  }
  
  /**
   * Beállítja a cellákban lévő értékek szineit.
   *
   * @param colors
   */
  public void repaintColors(DColors colors) {
    for (byte cand = 1; cand < 10; cand++) {
      candidate[cand].setForeground(colors.getColorElement(COLOR_CAND_BASE));
      candidate[cand].setBackground(colors.getColorElement(COLOR_CAND_SEL)); 
    }
    result.setForeground(colors.getColorElement(COLOR_RESULT));
  }

  /**
   * Kitörli az eredményt a cellából, majd bekapcsolja a lehetőségértékeket, de
   * mindet, ezért az aktuális sor, oszlop és blokk alapján át kell frissíteni.
   */
  public void clearResult() {
    setResult((byte) 0);
    for (byte cand = 1; cand < 10; cand++) {
      setCandidateState("" + cand, S_ON);
    }
  }

  /**
   * Visszaadja, hogy a lehetőségérték be van-e kapcsolva.
   *
   * @param cand lehetőség érték
   * @return állapot
   */
  public boolean isCandidateStateOn(byte cand) {
    return (candidate[cand].isVisible());
  }

  /**
   * Visszaadja, hogy a lehetőségérték be van-e kapcsolva.
   *
   * @param cand lehetőség érték
   * @return állapot
   */
  public boolean isCandidateStateOn(String cand) {
    if (cand.isEmpty()) {
      cand = "0";
    }
    return (Cell.this.isCandidateStateOn(Byte.parseByte(cand)));
  }

  /**
   * Lekérdezi, hogy hány lehetőségérték van bekapcsolva.
   *
   * @return lehetőség értékek száma
   */
  public byte getCandidatesCounter() {
    return (candidatesCounter);
  }

  /**
   * Visszaadja, hogy a cella meg van-e fejtve.
   *
   * @return
   */
  public boolean isResultStateOn() {
    return (!result.getText().isEmpty());
  }

  /**
   * Visszaadja az eresményben tárolt értéket, ha üres értékkel tér vissza, a
   * cella nincs megfejtve.
   *
   * @return
   */
  public String getResult() {
    String resultValue = "";
    if (isResultStateOn()) {
      resultValue = this.result.getText();
    }
    return (resultValue);
  }

  /**
   * Visszaadja a lehetőségértékeket egy sztringben.
   *
   * @return
   */
  public String getValidCandidates() {
    String candidates = "";
    for (byte cand = 1; cand < 10; cand++) {
      if (isCandidateStateOn(cand)) {
        candidates += cand;
      }
    }
    return candidates;
  }

  /**
   * Újraszámoltatja a jelöltek számát
   */
  public void recountCandidates() {
    byte counter = 0;
    for (byte cand = 1; cand < 10; cand++) {
      if (Cell.this.isCandidateStateOn(cand)) {
        counter++;
      }
    }
    setCandidatesCounter(counter);
  }

  /**
   * Beállítja a lehetőségérték jelöltségi állapotát.
   *
   * @param cand lehetőségérték
   * @param state állapot
   */
  public void setCandidateMark(String cand, boolean state) {
    setCandidateMark(Byte.parseByte(cand), state);
  }

  /**
   * Beállítja a lehetőségérték jelöltségi állapotát.
   *
   * @param cand lehetőségérték
   * @param state állapot
   */
  public void setCandidateMark(byte cand, boolean state) {
    candidate[cand].setOpaque(state);
    candidate[cand].repaint();
  }

  /**
   * A lehetőségérték állapotát beállítja, azaz ki/bekapcsolja.
   *
   * @param cand lehetőségérték
   * @param state állapot
   */
  public void setCandidateState(byte cand, boolean state) {
    if (isCandidateStateOn(cand) != state) {
      candidate[cand].setVisible(state);
      if (state == S_ON) {
        incCandidatesCounter();
      } else {
        decCandidatesCounter();
      }
    }
  }

  /**
   * A lehetőségérték állapotát beállítja, azaz ki/bekapcsolja.
   *
   * @param cand lehetőségérték
   * @param state állapot
   */
  public void setCandidateState(String cand, boolean state) {
    setCandidateState(Byte.parseByte(cand), state);
  }

  /**
   * Felülírja, hogy hány lehetőségérték van bekapcsolva.
   *
   * @param counter lehetőségértékek száma
   */
  public void setCandidatesCounter(byte counter) {
    candidatesCounter = counter;
  }

  /**
   * Beállítja a cella értékeit.
   * 
   * @param cell 
   */
  public void setCell(String cell) {
    if (cell.isEmpty()) {
      cell = ALLCANDIDATES;
    }
    if (cell.substring(0,1).equals("{")) {
      setResult("");
      for (byte cand = 1; cand < 10; cand++) {
        setCandidateState(cand, (cell.contains("" + cand)));
      }
    } else {
      setResult(cell);
    }
  }

  /**
   * A számot, mint eredményt jeleníti meg.
   * Eredmény beálításakor kikapcsolja a lehetőség-értékeket.
   * Eredmény törlésekor viszont el kell végezni a lehetőség-értékek beállítását.
   *
   * @param candidate lehetőségérték
   */
  public void setResult(byte candidate) {
    if (candidate > 0) {
      result.setText("" + candidate);
      result.setVisible(S_ON);
      for (byte cand = 1; cand < 10; cand++) {
        setCandidateState("" + cand, S_OFF);
      }
    } else {
      result.setText("");
      result.setVisible(S_OFF);
    }
  }

  /**
   * A számot, mint eredményt jeleníti meg.
   *
   * @param candidate lehetőségérték
   */
  public void setResult(String candidate) {
    String cCand;
    cCand = (candidate + "0").substring(0, 1);
    if (!ALLCANDIDATES.contains(cCand)) {
      cCand = "0";
    }
    setResult(Byte.parseByte(cCand));
  }

  /**
   * Csökkenti a jelöltértékek számlálóját.
   */
  public void decCandidatesCounter() {
    candidatesCounter--;
  }

  /**
   * Növeli a jelöltértékek számlálóját.
   */
  public void incCandidatesCounter() {
    candidatesCounter++;
  }

  /**
   * A cella rácson belüli pozícióját adja vissza.
   * 
   * @return
   */
  public DCoord getCoord() {
    return coord;
  }

  /**
   * A cella rácson belüli pozícióját állítja be.
   * 
   * @param coord A cella rácson belüli pozíciója 
   */
  public void setCoord(DCoord coord) {
    this.coord = coord;
  }

  /**
   * Visszaadja a cella bekapcsolt lehetőségértékeit vagy az eredmény értéket.
   * A { } közötti értékek a lehetőségértékeket jelölik.
   *
   * @return
   */
  @Override
  public String toString() {
    String content = getResult();
    if (!isResultStateOn()) {
      content = "{" + getValidCandidates() + "}";
    }
    return content;
  }

}
