/*
 * Szudoku program, amely a GDF részére diplomamunkául szolgál.
 * A programot írta: Pataki István, 2016-ban.
 */
package SudokuPIPkg.Lists;

import SudokuPIPkg.Basis.IConst;
import SudokuPIPkg.DataForms.DCoord;
import SudokuPIPkg.DataForms.DNumber;
import java.util.ArrayList;

public class Lists {

  /**
   * A keresés eredményéül kapott koordináták tárolóhelye
   */
  private ArrayList<LAssist> assistList = new ArrayList<>();

  /**
   * Akeresés eredményéül kapott koordináták és más adatok tárolóhelye
   */
  private ArrayList<LResult> resultList = new ArrayList<>();

  /**
   * A lépések kezdetének pozíciói a Listákban 0: Asssist, 1:LResult
   */
  private short[][] stepPosition = new short[IConst.MAX_STEPS_IN_LISTS][2];

  /**
   * A lépések számának tárolója
   */
  private short stepCount = 0;

  /**
   * Aktuális lépés sorszámának tárolója
   */
  private short stepOrder = 0;

  /**
   * Rekord hozzáadása a segédlistához
   *
   * @param stepOrder lépés sorszáma
   * @param coord
   * @param cellType Mit jelöl: C_RESULT = eredményt tartalmazó cella, C_ASSIST = segédcellák
   */
  public void addAssist(short stepOrder, DCoord coord, boolean cellType) {
    assistList.add(new LAssist(stepOrder, coord, cellType));
  }

  /**
   * Rekord hozzáadása az eredménylistához
   *
   * @param stepOrder lépés sorszáma
   * @param ruleOrder az eredmény szabály sorszáma
   * @param value érték
   * @param valueType érték típusa (RESULT-POSSIBILITY)
   * @param comment
   */
  public void addResult(short stepOrder, byte ruleOrder, DNumber value, boolean valueType, String comment) {
    resultList.add(new LResult(stepOrder, ruleOrder, value, valueType, comment));
  }

  /**
   * Törli a segédlistát
   */
  public void clearAssistList() {
    assistList.clear();
    for (short c = 0; c < IConst.MAX_STEPS_IN_LISTS; c++) {
      setAssistPosition(c, (short) 0);
    }
  }

  /**
   * Törli az eredménylistát
   */
  public void clearResultList() {
    resultList.clear();
    for (short c = 0; c < IConst.MAX_STEPS_IN_LISTS; c++) {
      setResultPosition(c, (short) 0);
    }
  }

  /**
   * Törli a segéd és az eredménylistát
   */
  public void clearAssistAndResultLists() {
    clearAssistList();
    clearResultList();
    setStepCount((short) 0);
    setStepOrder((short) 1);
  }

  /**
   * Növeli a lépés sorszámát
   */
  public void decStepOrder() {
    if (getStepOrder() > 1) {
      setStepOrder((short) (getStepOrder() - 1));
    }
  }

  /**
   * Lekérdezi a segéd oszloppozícióját
   *
   * @param index a rekord indexe
   * @return
   */
  public DCoord getAssistCoord(short index) {
    return assistList.get(index).getCoord();
  }

  /**
   * Lekérdezi, hogy az adott lépés esetén mi a kezdő pozíció a segédlistában.
   *
   * @param stepOrder lépés sorszáma
   * @return
   */
  public short getAssistPosition(short stepOrder) {
    return stepPosition[stepOrder][0];
  }

  /**
   * A segédlistában tárolt rekordok számát adja vissza
   *
   * @return
   */
  public short getAssistSize() {
    return (short) assistList.size();
  }

    /**
   * Az érték típusa (RESULT-POSSIBILITY)
   *
   * @param index a rekord indexe
   * @return
   */
  public boolean isAssistTypeResult(short index) {
    return assistList.get(index).isTypeResult();
  }

  /**
   * Lekérdezi a segédlistában a lépéssorszámot
   *
   * @param index a rekord indexe
   * @return
   */
  public short getAssistStepOrder(short index) {
    return assistList.get(index).getStepOrder();
  }

  /**
   * Lekérdezi az eredmény oszloppozícióját
   *
   * @param index a rekord indexe
   * @return
   */
  public DCoord getResultCoord(short index) {
    return resultList.get(index).getCoord();
  }

  /**
   * Lekérdezi, hogy az adott lépés esetén mi a kezdő pozíció az
   * eredménylistában.
   *
   * @param stepOrder
   * @return
   */
  public short getResultPosition(short stepOrder) {
    return stepPosition[stepOrder][1];
  }

  /**
   * Lekérdezi az eredmény megoldás szövegét
   *
   * @param ruleOrder a szabály sorszáma
   * @return
   */
  public String getRuleNameByOrder(short ruleOrder) {
    return IConst.RULESSHOW[ruleOrder];
  }
  
  /**
   * Lekérdezi az eredmény szabály sorszámát
   *
   * @param index a rekord indexe
   * @return
   */
  public byte getResultRuleOrder(short index) {
    return resultList.get(index).getRuleOrder();
  }

  /**
   * Az eredménylistában tárolt rekordok számát adja vissza.
   *
   * @return
   */
  public short getResultSize() {
    return (short) resultList.size();
  }

  /**
   * Az eredménylista adott lépés sorszámát adja vissza
   *
   * @param index a rekord indexe
   * @return
   */
  public short getResultStepOrder(short index) {
    return resultList.get(index).getStepOrder();
  }

  /**
   * Resulthoz tartozőó érték lekérdezése
   *
   * @param index a rekord indexe
   * @return
   */
  public byte getResultValue(short index) {
    return resultList.get(index).getValue();
  }

  /**
   * Az érték típusa (RESULT-POSSIBILITY)
   *
   * @param index a rekord indexe
   * @return
   */
  public boolean isResultTypeResult(short index) {
    return resultList.get(index).isTypeResult();
  }

  /**
   * A megjegyzés szövegét adja vissza.
   * 
   * @param index
   * @return 
   */
  public String getResultShowComment(short index) {
    return resultList.get(index).showComment();
  }

  /**
   * A lépések számát adja vissza
   *
   * @return
   */
  public short getStepCount() {
    return (stepCount);
  }

  /**
   * Az aktuális lépés sorszámát adja vissza
   *
   * @return
   */
  public short getStepOrder() {
    return stepOrder;
  }

  /**
   * Növeli a lépések számát
   */
  public void incStepCount() {
    setStepCount((short) (getStepCount() + 1));
  }

  /**
   * Növeli a lépés sorszámát, maximum a lista végéig
   */
  public void incStepOrder() {
    if (!isActualStepLast()) {
      setStepOrder((short) (getStepOrder() + 1));
    }
  }

  public boolean isActualStepLast() {
    return getStepOrder() > getStepCount();
  }

  /**
   * Beállítja, hogy a segédlistában az aktuális lépés honnantól kezdődik.
   *
   * @param stepOrder lépés sorszáma
   * @param position a listában elfoglalt kezdőpozíció
   */
  public void setAssistPosition(short stepOrder, short position) {
    stepPosition[stepOrder][0] = position;
  }

  /**
   * Beállítja, hogy az eredménylistában az aktuális lépés honnantól kezdődik.
   *
   * @param stepOrder lépés sorszáma
   * @param position a listában elfoglalt kezdőpozíció
   */
  public void setResultPosition(short stepOrder, short position) {
    stepPosition[stepOrder][1] = position;
  }

  /**
   * A lépésszámláló értékét álítja be
   *
   * @param value lépésszám
   */
  public void setStepCount(short value) {
    stepCount = value;
  }

  /**
   * Az aktuális lépés sorszámát állítja be
   *
   * @param stepOrder
   */
  public void setStepOrder(short stepOrder) {
    this.stepOrder = stepOrder;
  }

  @Override
  public String toString() {
    String res = "Üres listák";
    if ((getAssistSize() > 0) || (getResultSize() > 0)) {
      res = "Assist:\n=======\n";
      for (LAssist element : assistList) {
        res += element.toString() + "\n";
      }
      res += "\n\nResult:\n=======\n";
      for (LResult element : resultList) {
        res += element.toString() + "\n";
      }
    }
    return res;
  }
}
